﻿Imports ThinkGeo.MapSuite.Core
Imports System.Collections.ObjectModel

' Mark the class Serializable so that it workes in SQL Server state
' and in every serialization context
<Serializable()> _
Public Class SizedPointStyle
    Inherits Style
    Private m_pointStyle As PointStyle
    Private m_ratio As Single
    Private m_sizeColumnName As String

    Public Sub New()
        Me.New(New PointStyle(), String.Empty, 1)
    End Sub

    Public Sub New(ByVal pointStyle As PointStyle, ByVal sizeColumnName As String, ByVal ratio As Single)
        Me.m_pointStyle = pointStyle
        Me.m_sizeColumnName = sizeColumnName
        Me.m_ratio = ratio
    End Sub

    Public Property PointStyle() As PointStyle
        Get
            Return m_pointStyle
        End Get
        Set(ByVal value As PointStyle)
            m_pointStyle = value
        End Set
    End Property

    Public Property Ratio() As Single
        Get
            Return m_ratio
        End Get
        Set(ByVal value As Single)
            m_ratio = value
        End Set
    End Property

    Public Property SizeColumnName() As String
        Get
            Return m_sizeColumnName
        End Get
        Set(ByVal value As String)
            m_sizeColumnName = value
        End Set
    End Property

    Protected Overrides Sub DrawCore(ByVal features As IEnumerable(Of Feature), ByVal canvas As GeoCanvas, ByVal labelsInThisLayer As Collection(Of SimpleCandidate), ByVal labelsInAllLayers As Collection(Of SimpleCandidate))
        ' Loop through each feaure and determine how large the point should 
        ' be then adjust it's size.
        For Each feature As Feature In features
            Dim sizeData As Single = Convert.ToSingle(feature.ColumnValues(m_sizeColumnName))
            Dim symbolSize As Single = sizeData / m_ratio
            m_pointStyle.SymbolSize = symbolSize
            Dim drawFeatures As New Collection(Of Feature)()
            drawFeatures.Add(feature)
            m_pointStyle.Draw(drawFeatures, canvas, labelsInThisLayer, labelsInAllLayers)
        Next
    End Sub

    Protected Overrides Function GetRequiredColumnNamesCore() As Collection(Of String)
        ' Here we grab the columns from the pointStyle and then add
        ' the sizeColumn name to make sure we pull back the column
        '  that we need to calculate the size
        Dim columns As New Collection(Of String)()
        columns = m_pointStyle.GetRequiredColumnNames()
        If Not columns.Contains(m_sizeColumnName) Then
            columns.Add(m_sizeColumnName)
        End If

        Return columns
    End Function

    Protected Overrides Sub DrawSampleCore(ByVal canvas As GeoCanvas)
        ' Here we simply pass the call to the pointStyle's DrawSampleCore
        m_pointStyle.DrawSample(canvas)
    End Sub
End Class
